@@ -1,12 +1,10 @@
 import argparse
 import os
-import re
 import sys
 import time
 from argparse import Namespace
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple
-from urllib import parse
 
 import pkg_resources
 import websocket
@@ -18,11 +16,12 @@ from rich.prompt import Confirm
 from websocket import WebSocketApp
 
 from dstack import providers
-from dstack.api.backend import DEFAULT, DEFAULT_REMOTE, get_backend_by_name
+from dstack.api.backend import get_backend_by_name, get_current_remote_backend, get_local_backend
 from dstack.api.logs import poll_logs
 from dstack.api.repo import load_repo_data
 from dstack.api.run import list_runs_with_merged_backends
 from dstack.backend.base import Backend
+from dstack.backend.base.logs import fix_urls
 from dstack.cli.commands import BasicCommand
 from dstack.cli.common import console, print_runs
 from dstack.core.error import check_backend, check_config, check_git
@@ -91,25 +90,9 @@ def poll_logs_ws(backend: Backend, repo_address: RepoAddress, job_head: JobHead)
     job = backend.get_job(repo_address, job_head.job_id)
 
     def on_message(ws: WebSocketApp, message):
-        pat = re.compile(
-            f"http://(localhost|0.0.0.0|127.0.0.1|{job.host_name}):[\\S]*[^(.+)\\s\\n\\r]"
-        )
-        if re.search(pat, message):
-            if job.host_name and job.ports and job.app_specs:
-                for app_spec in job.app_specs:
-                    port = job.ports[app_spec.port_index]
-                    url_path = app_spec.url_path or ""
-                    url_query_params = app_spec.url_query_params
-                    url_query = (
-                        ("?" + parse.urlencode(url_query_params)) if url_query_params else ""
-                    )
-                    app_url = f"http://{job.host_name}:{port}"
-                    if url_path or url_query_params:
-                        app_url += "/"
-                        if url_query_params:
-                            app_url += url_query
-                    message = re.sub(pat, app_url, message)
-        sys.stdout.write(message)
+        message = fix_urls(message, job)
+        sys.stdout.buffer.write(message)
+        sys.stdout.buffer.flush()
 
     def on_error(_: WebSocketApp, err: Exception):
         if isinstance(err, KeyboardInterrupt):
@@ -117,8 +100,9 @@ def poll_logs_ws(backend: Backend, repo_address: RepoAddress, job_head: JobHead)
             if Confirm.ask(f"\n [red]Abort the run '{run_name}'?[/]"):
                 backend.stop_jobs(repo_address, run_name, abort=True)
                 console.print(f"[grey58]OK[/]")
+            exit()
         else:
-            console.print(str(err))
+            console.print(err)
 
     def on_open(_: WebSocketApp):
         pass
@@ -141,9 +125,7 @@ def poll_logs_ws(backend: Backend, repo_address: RepoAddress, job_head: JobHead)
     try:
         while True:
             _job_head = backend.get_job(repo_address, job_head.job_id)
-            run = next(
-                iter(backend.get_run_heads(repo_address, [_job_head], include_request_heads=False))
-            )
+            run = backend.list_run_heads(repo_address, _job_head.run_name)[0]
             if run.status.is_finished():
                 break
             time.sleep(POLL_FINISHED_STATE_RATE_SECS)
@@ -154,6 +136,7 @@ def poll_logs_ws(backend: Backend, repo_address: RepoAddress, job_head: JobHead)
 
 
 def poll_run(repo_address: RepoAddress, job_heads: List[JobHead], backend: Backend):
+    run_name = job_heads[0].run_name
     try:
         console.print()
         request_errors_printed = False
@@ -166,10 +149,10 @@ def poll_run(repo_address: RepoAddress, job_heads: List[JobHead], backend: Backe
             task = progress.add_task("Provisioning... It may take up to a minute.", total=None)
             while True:
                 time.sleep(POLL_PROVISION_RATE_SECS)
-                _job_heads = [
-                    backend.get_job(repo_address, job_head.job_id) for job_head in job_heads
-                ]
-                run = next(iter(backend.get_run_heads(repo_address, _job_heads)))
+                run_heads = backend.list_run_heads(repo_address, run_name)
+                if len(run_heads) == 0:
+                    continue
+                run = run_heads[0]
                 if run.status == JobStatus.DOWNLOADING and not downloading:
                     progress.update(task, description="Downloading deps... It may take a while.")
                     downloading = True
@@ -210,7 +193,6 @@ def poll_run(repo_address: RepoAddress, job_heads: List[JobHead], backend: Backe
                 from_run=True,
             )
     except KeyboardInterrupt:
-        run_name = job_heads[0].run_name
         if Confirm.ask(f" [red]Abort the run '{run_name}'?[/]"):
             backend.stop_jobs(repo_address, run_name, abort=True)
             console.print(f"[grey58]OK[/]")
@@ -272,13 +254,19 @@ class RunCommand(BasicCommand):
             exit(1)
         try:
             repo_data = load_repo_data()
-            backend_name = DEFAULT
+            backend = get_local_backend()
             if args.remote is not None:
                 if len(args.remote) == 0:
-                    backend_name = DEFAULT_REMOTE
+                    remote_backend = get_current_remote_backend()
+                    if remote_backend is None:
+                        console.print(f"No remote configured. Run `dstack config`.")
+                        exit(1)
                 else:
-                    backend_name = args.remote[0]
-            backend = get_backend_by_name(backend_name)
+                    remote_backend = get_backend_by_name(args.remote[0])
+                    if remote_backend is None:
+                        console.print(f"Backend '{args.remote[0]}' is not configured")
+                        exit(1)
+                backend = remote_backend
 
             (
                 provider_name,
@@ -297,11 +285,7 @@ class RunCommand(BasicCommand):
                 if args.tag_name:
                     tag_head = backend.get_tag_head(repo_data, args.tag_name)
                     if tag_head:
-                        # if args.yes or Confirm.ask(f"[red]The tag '{args.tag_name}' already exists. "
-                        #                            f"Do you want to override it?[/]"):
                         backend.delete_tag_head(repo_data, tag_head)
-                        # else:
-                        #     return
                 jobs = provider.submit_jobs(backend, args.tag_name)
                 backend.update_repo_last_run_at(
                     repo_data, last_run_at=int(round(time.time() * 1000))
@@ -310,7 +294,7 @@ class RunCommand(BasicCommand):
                 if not args.detach:
                     poll_run(repo_data, jobs, backend)
             else:
-                sys.exit(f"Call 'dstack init' first")
+                sys.exit(f"Call `dstack init` first")
         except ValidationError as e:
             sys.exit(
                 f"There a syntax error in one of the files inside the {os.getcwd()}/.dstack/workflows directory:\n\n{e}"
